% =========================================================================
% This file provides an example of a simulation of a nonlinear ordinary 
% differential equation (ODE). The example system ist the nonlinear 
% tractor model described in the lecture notes. 
% 
% AUTHOR: Joerg Fischer, Lukas Klar
% Systems control and optimization laboratory, 
% Department of Microsystems Engineering (IMTEK),
% University of Freiburg
% 
% File Created: 18.04.2016
% =========================================================================
%
% 
% Description of the tractor example:
% States of the tractor are the x-position (X), the y-position (Y) and the
% orientation angle (beta) according to the x-axis
%     x = [X(t); Y(t); beta(t)] 
% 
% Control input u of the system is the steering angle and the output is the
% distance from the x-axis aka y-position.
% 
% In the following, the tractor is simulated for 50 seconds. Starting with 
% the state [0; 5; 0] and a sinusoidal input.

%###############  0.  Closing all open figures
close all

%###############  1.  Definition of ODE dx/dt = f(x,u)
% This is implemented in the file: tractor_ode_nl.m

%###############  2.  Definition of output function y = g(x,u)
% This is implemented in the file: tractor_out.m
  
%###############  3.  Setting Simulation time parameters
% lenght of a single integration step in seconds
stepTime = 0.1;
% number of time steps performed in simulation
numSteps = 500;
% --> total simulated time is stepTime * numSteps = 50s

%###############  4.  Definition of initial state
x_0 = [0; 5; 0];

%###############  5.  Definition of control input trajectory
% U is a sequence of control inputs u, where u is a vector if the 
% system has more than one inpput. The column index of U represents the 
% time index when a particular u (rows of U) is applied to the system.
%
% Create a sinusoidal input sequence (2 periods with amplitude 0.05)
U = 0.05 * sin(linspace(0, 4*pi, numSteps));

%###############  6.  Simulation of system
% Input Parameter: see steps 1 to 4 above and documentation of nlsim
% Output Parameter: X, Y are the state and output trajectory, 
%                  T is the timegrid belonging to X and Y
[X, Y, T] = nlsim(@tractor_ode_nl, x_0, U, stepTime, @tractor_out);

%###############  7.  Plotting output
figure;
plot(T, Y)

%###############  8. Legend and labels
title('Ausgangssignal des gesteuerten Traktors');
legend('Y_{nonlin}', 'Location', 'NORTHWEST');
xlabel('Zeit in s');
ylabel('Ort in m');

